@@ -1,11 +1,32 @@
+"""
+Требования: 
+в ините должна содержаться информация о 
+    device, 
+    random_state 
+    
+Также должно быть метод
+    run_epoch
+    
+"""
+
 from torch import nn, optim
 import torch
-device = torch.device("cuda:0" if torch.cuda.is_available() else "cpu")
 import numpy as np
+import random
 
+def set_determenistic(seed=None,precision=10):
+    if seed is None:
+        return
+    np.random.seed(seed)
+    random.seed(seed)
+    torch.backends.cudnn.benchmark = False
+    torch.backends.cudnn.deterministic = True
+    torch.manual_seed(seed)
+    torch.set_printoptions(precision=precision)
 
 class SimpleLSTM(nn.Module):
-    def __init__(self, in_features, n_hidden, n_layers=2, bidirectional=False, dropout=0.5):
+    def __init__(self, in_features, n_hidden, n_layers=3, bidirectional=False, dropout=0.5, device=torch.device('cpu'), seed=None):
+        set_determenistic(seed)
         super(SimpleLSTM, self).__init__()
         self.n_hidden = n_hidden
         self.n_layers = n_layers
@@ -19,12 +40,14 @@ class SimpleLSTM(nn.Module):
         self.k_bidir = 2 if bidirectional else 1 
 
         self.linear = nn.Linear(in_features=n_hidden, out_features=in_features)
+        self.device = device
+        
     
     
     def initHidden(self,batch_size):
         self.hidden = (
-            torch.zeros(self.n_layers*self.k_bidir, batch_size, self.n_hidden).to(device),
-            torch.zeros(self.n_layers*self.k_bidir, batch_size, self.n_hidden).to(device)
+            torch.zeros(self.n_layers*self.k_bidir, batch_size, self.n_hidden).to(self.device),
+            torch.zeros(self.n_layers*self.k_bidir, batch_size, self.n_hidden).to(self.device)
         )
     def forward(self, sequences):
         batch_size  = len(sequences)
@@ -61,8 +84,8 @@ class SimpleLSTM(nn.Module):
                 x,y = np.array(x),np.array(y) #df.index rif of
                 self.initHidden(x.shape[0])
                 
-                x = torch.tensor(x).float().to(device).requires_grad_()
-                y_true = torch.tensor(y).float().to(device)
+                x = torch.tensor(x).float().to(self.device).requires_grad_()
+                y_true = torch.tensor(y).float().to(self.device)
                 y_pred = self.forward(x).unsqueeze(1)
                 y_pred = forecast_multistep(y_pred,points_ahead)
                 
