@@ -1,3 +1,8 @@
+# косяк, мой подсчет метрик не работает если там нет трушных 1
+"""
+CDSDSDS
+"""
+
 from sklearn.model_selection import train_test_split
 import numpy as np
 import pandas as pd
@@ -10,27 +15,59 @@ def ts_train_test_split(df, len_seq,
                      points_ahead=1, gap=0, shag=1, intersection=True,
                      test_size=None,train_size=None, random_state=None, shuffle=True,stratify=None):
     """
-	df - требование, но если тебе не хочется то просто сделай np.array на выходе и все
-    Разбить временные ряды на трейн и тест
-    len_seq- количество времменых точек в трейн
-    points_ahead - количество времменых точек в прогнозе
-    gap - расстояние между концом трейна и началом теста
-    intersection - если нет, то в выборке нет перескающих множеств (временнызх моментов)
-    shag - через сколько прыгаем
-    train_size - float от 0 до 1
-    
-    return list of dfs
-    
+    Функция которая разбивает временной ряд на трейн и тест выборки 
     
+    Временной ряд здесь это вообще вся история
+    Функционал позволяет разбивать ....
+
+    Parameters
+    ----------
+    df : pd.DataFrame
+        Array of shape (n_samples, n_features) with timestamp index.
+    len_seq : int 
+        Длина 
+    flag : bool (optional, default=True)
+        If true, then do one thing.
+        If false, then do another thing.
+    f : callable (optional, default=None)
+        Call-back function.  If not specified, then some other function
+        will be used
+    **kwargs :
+        Additional keyword arguments will be passed to name_of_function
     
+    Returns
+    -------
+    z : ndarray
+        result of shape (n_samples,).  Note that here we use "ndarray" rather
+        than "array_like", because we assure we'll return a numpy array.
     
+    Examples
+    --------
+    >>> X = np.ones((4, 3))
+    >>> y = np.ones(4)
+    >>> sklearn_template(X, y)
+    (z, xmin, xmax)  # this should match the actual output
     """
+
+#    """
+#	df - требование, но если тебе не хочется то просто сделай np.array на выходе и все
+#    Разбить временные ряды на трейн и тест
+#    len_seq- количество времменых точек в трейн
+#    points_ahead - количество времменых точек в прогнозе
+#    gap - расстояние между концом трейна и началом теста
+#    intersection - если нет, то в выборке нет перескающих множеств (временнызх моментов)
+#    shag - через сколько прыгаем
+#    train_size - float от 0 до 1
+#    
+#    return list of dfs
+#    
+#    """
     #TODO требования к входным данным прописать
     #TODO переписать энергоэффективно чтобы было
     #TODO пока временные характеристики int_ами пора бы в pd.TimdDelta
     # нет индексов 
-    assert len_seq + points_ahead + gap + shag-1 <= len(df)
-    how='seq to seq'
+    assert len_seq + points_ahead + gap + 1 <= len(df)
+    how='seq to seq'   
 
 # -------------------------------------------------------  
 #             
@@ -48,7 +85,7 @@ def ts_train_test_split(df, len_seq,
             return x_start + shag
     else:
         def compute_new_x_start(x_start,y_end,shag):
-            y_end + shag
+            return y_end + shag
     
     X = []
     y = []
@@ -62,11 +99,13 @@ def ts_train_test_split(df, len_seq,
         y_end = y_start + points_ahead
           
     
-    if test_size==0:
+    if (test_size==0) | (len(X)==1):
         indices = np.array(range(len(X)))
         #             np.random.seed(random_state)
         if shuffle:
+            print(indices)
             np.random.shuffle(indices)
+            print(indices)
         X = [X[i] for i in indices]
         y = [y[i] for i in indices]
         return X,[],y,[]
@@ -151,6 +190,7 @@ def split_by_repeated(series,df=None):
     else:
         return result
         
+
 def evaluating_change_point(true, prediction, metric='nab', 
                             numenta_time=None,
                             portion=0.1,
@@ -162,8 +202,10 @@ def evaluating_change_point(true, prediction, metric='nab',
                             hidden_anomalies_mode = True,
                             plot_figure=False,
                             change_point_mode = True,
+                            verbose=True
                             ):
     """
+    
     true - both:
                 list of pandas Series with binary int labels
                 pandas Series with binary int labels
@@ -172,6 +214,9 @@ def evaluating_change_point(true, prediction, metric='nab',
                       pandas Series with binary int labels
     metric: 'nab', 'binary' (FAR, MAR), 'average_delay'. Default='nab'
     
+    verbose:  : booleant, default=True.
+                  Если True, то вывод полезной информации в output
+    
     <<< For 'nab', 'average_delay' metrics: ...
         portion : float, default=0.1
                 Параметр нужен, если numenta_time=None
@@ -182,7 +227,7 @@ def evaluating_change_point(true, prediction, metric='nab',
                 Параметр размещения scoring window относительно аномалии. 
                 'left'  : scoring window будет по левую сторону от аномалии
                 'right' : scoring window будет по правую сторону от аномалии
-                'left'  : scoring window будет по размещено относительно центра
+                'center': scoring window будет по размещено относительно центра
                           аномалии
 
         intersection_mode: 'cut left', 'cut right', 'both'. Defualt='right'
@@ -261,32 +306,53 @@ def evaluating_change_point(true, prediction, metric='nab',
         return f1
     
     def average_delay(detecting_boundaries, prediction):
+        """
+        Возвращает add, missing, all_true_anom
+        
+        detecting_boundaries - где правая граница трушная аномалия, только так
+        """
         
         def single_average_delay(detecting_boundaries, prediction):
             missing = 0
             detectHistory = []
-            for couple in detecting_boundaries:
-                t1 = couple[0]
-                t2 = couple[1]
+            all_true_anom = 0
+            _df_fill_bounds =  pd.DataFrame(np.ones((len(prediction),len(detecting_boundaries)))*np.nan,index=prediction.index)
+            for i in range(len(detecting_boundaries)):
+                t1 = detecting_boundaries[i][0]
+                t2 = detecting_boundaries[i][1]
+                _df_fill_bounds.loc[t1:t2,i]=1
                 if prediction[t1:t2].sum()==0:
                     missing+=1
                 else:
                     detectHistory.append(prediction[prediction ==1][t1:t2].index[0]-t1)
-            return missing, detectHistory
+                all_true_anom+=1
+            # FPs
+                        
+            
+                
+            ts_fp = pd.Series(np.ones(len(prediction)),index=prediction.index)
+            ts_fp.loc[_df_fill_bounds.dropna(how='all').index]=0
+            ts_fp = ts_fp * prediction
+            FP = ts_fp.sum()
+            return missing, detectHistory, FP, all_true_anom
             
         
         if type(prediction) != type(list()):
-            missing, detectHistory = single_average_delay(detecting_boundaries, prediction)
+            missing, detectHistory, FP, all_true_anom = single_average_delay(detecting_boundaries, prediction)
         else:
-            missing, detectHistory = 0, []
+            missing, detectHistory, FP, all_true_anom = 0, [], 0, 0
             for i in range(len(prediction)):
-                missing_, detectHistory_ = single_average_delay(detecting_boundaries[i], prediction[i])
-                missing, detectHistory = missing+missing_, detectHistory+detectHistory_
+                missing_, detectHistory_, FP_, all_true_anom_ = single_average_delay(detecting_boundaries[i], prediction[i])
+                missing, detectHistory, FP, all_true_anom = missing+missing_, detectHistory+detectHistory_, FP+FP_, all_true_anom+all_true_anom_
 
         add = pd.Series(detectHistory).mean()
-        print('Average delay', add)
-        print(f'A number of missed CPs = {missing}')
-        return add
+        if verbose:
+            print('Amount of true anomalies',all_true_anom)
+            print(f'A number of missed CPs = {missing}')
+            print(f'A number of FPs = {FP}')
+            print('Average delay', add)
+            
+        return add, missing, int(FP), all_true_anom
     
     def evaluate_nab(detecting_boundaries, prediction, 
                      table_of_coef=None,
@@ -385,10 +451,14 @@ def evaluating_change_point(true, prediction, metric='nab',
                                                             table_of_coef['A_fp']['Standart'],
                                                             koef=scale_koef),
                                            index = ts.index)
-                    plt.plot(ts_profile)
-                    for pred_anom in prediction.loc[ts.index[0]:ts.index[-1]].index:
-                        plt.axvline(pred_anom,c='r')
-                    plt.show()
+                    plt.plot(ts_profile,label='profile')
+                    ind_ = prediction[prediction==1].loc[ts.index[0]:ts.index[-1]].index
+                    if len(ind_)>0:
+                        for pred_anom in ind_:
+                            plt.axvline(pred_anom,c='r',label='predicted anomaly')
+                        plt.legend()
+                        plt.title(str(i))
+                        plt.show()
                 
             
             Scores, Scores_perfect, Scores_null=[], [], []
@@ -464,7 +534,9 @@ def evaluating_change_point(true, prediction, metric='nab',
         
 
     if not metric=='binary':
-        def single_detecting_boundaries(true, numenta_time, true_1_indexes,
+        if metric=='average_delay':
+            anomaly_window_destenation= 'left'
+        def single_detecting_boundaries(true, prediction, numenta_time, true_1_indexes,
                                         anomaly_window_destenation='left',portion=0.1):
             detecting_boundaries=[]
             td = pd.Timedelta(numenta_time) if numenta_time is not None else \
@@ -478,10 +550,39 @@ def evaluating_change_point(true, prediction, metric='nab',
                     anomaly_window_destenation.append([val - td/2, val + td/2])
                 else:
                     raise('choose anomaly_window_destenation')
+                    
+                    
+            # block for resolving intersection problem:
+            # важно не ошибиться, и всегда следить, чтобы везде правая граница далее
+            # не включалась, иначе будет пересечения окон             
+            new_detecting_boundaries = detecting_boundaries.copy()
+            if new_detecting_boundaries[0][0] < prediction.index[0]:
+                new_detecting_boundaries[0][0] = prediction.index[0]
+            if new_detecting_boundaries[-1][-1] > prediction.index[-1]:
+                new_detecting_boundaries[-1][-1] = prediction.index[-1]
+            for i in range(len(new_detecting_boundaries)-1):
+                if new_detecting_boundaries[i][1] >= new_detecting_boundaries[i+1][0]:
+                    print(f'Intersection of scoring windows{new_detecting_boundaries[i][1], new_detecting_boundaries[i+1][0]}')
+                    if intersection_mode == 'cut left':
+                        new_detecting_boundaries[i][1] = new_detecting_boundaries[i+1][0]
+                    elif intersection_mode == 'cut right':
+                        new_detecting_boundaries[i+1][0] = new_detecting_boundaries[i][1]
+                    elif intersection_mode == 'cut both':
+                        _a  = new_detecting_boundaries[i][1]
+                        new_detecting_boundaries[i][1] = new_detecting_boundaries[i+1][0]
+                        new_detecting_boundaries[i+1][0] = _a
+                    else:
+                        raise("choose the intersection_mode")
+            detecting_boundaries = new_detecting_boundaries.copy()        
+                    
+                    
+                    
+            
             return detecting_boundaries
         
         if type(true) != type(list()):
-            detecting_boundaries = single_detecting_boundaries(true=true, 
+            detecting_boundaries = single_detecting_boundaries(true=true,
+                                                               prediction=prediction, 
                                                                numenta_time=numenta_time, 
                                                                true_1_indexes=true_1_indexes,
                                                                anomaly_window_destenation=anomaly_window_destenation,
@@ -491,32 +592,12 @@ def evaluating_change_point(true, prediction, metric='nab',
             detecting_boundaries=[]
             for i in range(len(true)):
                 detecting_boundaries.append(single_detecting_boundaries(true=true[i], 
+                                                                        prediction=prediction[i],                                                                
                                                                         numenta_time=numenta_time, 
                                                                         true_1_indexes=true_1_indexes[i],
                                                                         anomaly_window_destenation=anomaly_window_destenation,
                                                                         portion=portion))
-        # block for resolving intersection problem:
-        # важно не ошибиться, и всегда следить, чтобы везде правая граница далее
-        # не включалась, иначе будет пересечения окон             
-        new_detecting_boundaries = detecting_boundaries.copy() 
-        if new_detecting_boundaries[0][0] < prediction.index[0]:
-            new_detecting_boundaries[0][0] = prediction.index[0]
-        if new_detecting_boundaries[-1][-1] > prediction.index[-1]:
-            new_detecting_boundaries[-1][-1] = prediction.index[-1]
-        for i in range(len(new_detecting_boundaries)-1):
-            if new_detecting_boundaries[i][1] >= new_detecting_boundaries[i+1][0]:
-                print("Intersection of scoring windows")
-                if intersection_mode == 'cut left':
-                    new_detecting_boundaries[i][1] = new_detecting_boundaries[i+1][0]
-                elif intersection_mode == 'cut right':
-                    new_detecting_boundaries[i+1][0] = new_detecting_boundaries[i][1]
-                elif intersection_mode == 'cut both':
-                    _a  = new_detecting_boundaries[i][1]
-                    new_detecting_boundaries[i][1] = new_detecting_boundaries[i+1][0]
-                    new_detecting_boundaries[i+1][0] = _a
-                else:
-                    raise("choose the intersection_mode")
-        detecting_boundaries = new_detecting_boundaries.copy()
+
 
     if metric== 'nab':
         return evaluate_nab(detecting_boundaries, 
@@ -531,4 +612,77 @@ def evaluating_change_point(true, prediction, metric='nab',
     elif metric=='average_delay':
         return average_delay(detecting_boundaries, prediction)
     elif metric== 'binary':
-        return binary(true, prediction)
\ No newline at end of file
+        return binary(true, prediction)
+        
+        
+def df2dfs(df,  # Авторы не рекомендуют так делать,
+            resample_freq = None, # требования
+            thereshold_gap = None, 
+            koef_freq_of_gap = 1.2, # 1.2 проблема которая возникает которую 02.09.2021 я написал в ИИ 
+            plot = True,
+            col = None):
+    """
+    Функция которая преообратает raw df до требований к входу на DL_AD:
+    
+    то есть разбивает df на лист of dfs by gaps 
+    
+    Не ресемлирует так как это тяжелая задача, но если частота реже чем 
+    koef_freq_of_gap of thereshold_gap то это воспринимается как пропуск. 
+    Основной посыл: если сигнал приходит чаще, то он не уползает сильно, 
+    а значит не приводит к аномалии, а если редко то приводит, поэтому воспри-
+    нимается как пропуск. 
+    
+    Parameters
+    ----------   
+    df : pd.DataFrame
+        Исходный временной ряд полностью за всю историю   
+        
+    resample_freq: pd.TimeDelta (optional, default=None)
+        Частота дискретизации временного ряда. 
+        Если default то самая частая частота дискретизации. При этом, 
+        если нет выраженной частоты вылетит ошибка. 
+    thereshold_gap : pd.TimeDelta (optional, default=None)
+        Порог периода, превышая который функция будет воспринимать
+        данный период как пропуск. 
+    koef_freq_of_gap : float or int (optional if thereshold_gap==None,
+        default=1.2)  
+        thereshold_gap = koef_freq_of_gap * resample_freq
+    plot : bool (optional, default=True)
+        If true, then отрисуется нарезка
+        If false, then не отрисуется нарезка   
+    col : int of str (optional, default=True)
+        Название или номер колонки для отрисовки
+        Если None первая колонка
+
+    Returns
+    -------
+    dfs : list of pd.DataFrame
+        Список времменных рядов без пропусков с относительно стабильной 
+        частотой дискретизации. 
+    """
+    
+    df = df.dropna(how='all').dropna(1,how='all')
+    dts  = df.dropna(how='all').index.to_series().diff()
+    if resample_freq is None:
+        dts_dist = dts.value_counts()
+        if dts_dist[0] > dts_dist[1:].sum():
+            resample_freq  = dts_dist.index[0]
+        else: 
+            print(dts_dist)
+            raise Exception("Необходимо самостоятельно обработать функцию так как нет преобладающей частоты дискретизации")
+    thereshold_gap = resample_freq*koef_freq_of_gap if thereshold_gap is None else thereshold_gap
+    gaps = (dts > thereshold_gap).astype(int).cumsum()
+    dfs = [df.loc[gaps[gaps==stage].index] for stage in gaps.unique()]
+
+    if plot:
+        f, ax = plt.subplots()
+        if col is None:
+            col = df.columns[0]
+        else:
+            if type(col)==type(int):
+                col = df.columns[col]
+        for df in dfs:
+            df[col].plot(ax=ax)
+    return dfs
+    
+    
\ No newline at end of file
